package net.pricefx.common.api.pa;

import net.pricefx.common.api.FieldFormatType;

public enum FieldType {
	NUMBER(true), 
	QUANTITY(true), 
	TEXT(false), 
	DATE(false), 
	MONEY(true), 
	CURRENCY(false),
	UOM(false),
	LOB(false),
	DATETIME(false),
	BOOLEAN(false),
	INTEGER(true);
	// INTEGER field is meant to be more like an enumerated type or an ID, and was introduced with numeric = false.
	// Meanwhile we opened it up and using it in numeric expressions is possible. It can still be set to be a Dimension,
	// making it available as groupBy field.

	private boolean numeric;

	private FieldType(boolean numeric) {
		this.numeric = numeric;
	}

	public boolean isNumeric() {
		return numeric;
	}

	public boolean isGroupable() {
		switch (this) {
		case TEXT:
		case DATE:
		case DATETIME:
		case CURRENCY:
		case UOM:
		case INTEGER:
			return true;
		default:
			return false;
		}
	}
	
	public DataType toDataType(){
		switch (this){
			case TEXT: 
			case UOM:
			case CURRENCY:
				return DataType.STRING; 
			case NUMBER:
			case MONEY:
			case QUANTITY:
				return DataType.NUMBER;
			case DATE: 
				return DataType.DATE;
			case DATETIME: 
				return DataType.DATETIME;
			case LOB: 
				return DataType.LOB;
			case BOOLEAN: 
				return DataType.BOOLEAN;
			case INTEGER: 
				return DataType.LONG;
		}
		throw new IllegalArgumentException("Cannot resolve DataType of " + this);
	}
	
	public static FieldType toType(final DataType dataType){
		if (dataType != null) switch (dataType){
			case BOOLEAN:			
			case STRING: 
				return FieldType.TEXT;
			case NUMBER: 
				return FieldType.NUMBER;
			case DATE: 
				return FieldType.DATE;
			case DATETIME: 
				return FieldType.DATETIME;
			case LOB: 
				return FieldType.LOB;
			case LONG:
				return FieldType.INTEGER;
			case UNKNOWN:
				// throw exception
		}
		throw new IllegalArgumentException("Unexpected DataType: " + dataType);		
	}

	public static FieldType toType(final FieldFormatType fieldFormatType){
	    if (fieldFormatType != null) switch(fieldFormatType) {
        case DATE:
            return FieldType.DATE;
        case DATETIME:
            return FieldType.DATETIME;
        case INTEGER:
            return FieldType.INTEGER;
        case MONEY:
        case MONEY_CHF:
        case MONEY_EUR:
        case MONEY_GBP:
        case MONEY_JPY:
        case MONEY_PLN:
        case MONEY_USD:
            return FieldType.MONEY;
        case NUMERIC:
        case NUMERIC_LONG:
        case PERCENT:
            return FieldType.NUMBER;
        case TEXT:
            return FieldType.TEXT;
        case LINK:
            // to be stored as text bu using LOB as this may get quite big
        case COMPLEX:
            // to be stored as CLOB (json representation)
            return FieldType.LOB;
        default:
            // throw exception
        }
	    throw new IllegalArgumentException("Unexpected FieldFormatType: " + fieldFormatType);     
	}
	
}
