package net.pricefx.common.apibuilder.calculationflowaction;

import net.pricefx.common.apibuilder.Invocations;

/**
 * This action starts calculation of selected rebate record set records. It's a
 * programmatic version of the same action available in UI. As such it allows
 * user to schedule the action in calculation flow formula.
 */
public class ObjRefAction<ATTACHEDTO extends Help> extends Action {

	private String partition;
	private String typeCode;
	private Object filter;
	
	private ATTACHEDTO backLink;
	
	public ObjRefAction(ATTACHEDTO backLink, Invocations<MethodInvocationRecord> invocations) {
		super(invocations);
		this.backLink = backLink;
	}

	/**
	 * @param partition partition the object references will be renewed for.
	 */
	public ObjRefAction<ATTACHEDTO> setPartition(String partition) {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".setPartition", partition));
		this.partition = partition;
		return this;
	}

	/**
	 * @param typeCode
	 *            type code of domain object the object references will be
	 *            renewed for.
	 */
	public ObjRefAction<ATTACHEDTO> setTypeCode(String typeCode) {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".setTypeCode", typeCode));
		this.typeCode = typeCode;
		return this;
	}
	
	/**
	 * @param filter
	 *            restricts object references renewal to a subset defined by
	 *            filter.
	 */
	public ObjRefAction<ATTACHEDTO> setFilter(Object filter) {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".setFilter", filter));
		this.filter = filter;
		return this;
	}

	public ATTACHEDTO back() {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".back"));
		return backLink;
	}

	@Override
	public String getObjectName() {
		return backLink.getObjectName() + ".ObjRefAction";
	}

	public String getPartition() { return partition;	}
	public String getTypeCode() { return typeCode; }
	public Object getFilter() { return filter; }
	
	@Override public String toString() { 
		return "ObjRefAction [partition=" + partition + ", typeCode=" + typeCode + ", filter=" + filter + ", hashCode()=" + hashCode() + "]";
	}

}