package net.pricefx.domain;

import java.util.EnumSet;
import java.util.HashMap;
import java.util.Map;


import net.pricefx.common.api.FieldFormatType;


public class LookupTable extends TimedObjectWithStatus {

	public enum LookupTableType {
		SIMPLE, RANGE, MATRIX, SIMPLE_INT_KEY, SIMPLE_REAL_KEY, SIMPLE_DATE_KEY, JSON;
		
		private static final Map<String,LookupTableType> lookup = new HashMap<String,LookupTableType>();
		
		static {
		    for(LookupTableType ct : EnumSet.allOf(LookupTableType.class)){
		         lookup.put(ct.name(), ct);
		    }
		}	
		
	    public static LookupTableType get(String code) { 
	        if(code == null) return null;	        
	       	return lookup.get(code.toUpperCase()); 
	    }			
	}
	
	
	public enum LookupTableValueType {
		STRING, INT, REAL, MATRIX, DATE, MATRIX2, MATRIX3, MATRIX4, MATRIX5, MATRIX6, JSON, JSON2;
		
		private static final Map<String,LookupTableValueType> lookup = new HashMap<String,LookupTableValueType>();
		
		static {
		    for(LookupTableValueType ct : EnumSet.allOf(LookupTableValueType.class)){
		         lookup.put(ct.name(), ct);
		    }
		}	
		
	    public static LookupTableValueType get(String code) { 
	        if(code == null) return null;	        
	       	return lookup.get(code.toUpperCase()); 
	    }			
	}	
	
	private LookupTableType type;
	
	private LookupTableValueType valueType;
	
	private Long nodeId;

	private String userGroupEdit;
	
	private String userGroupViewDetails;
	
	private Boolean hideWarnings;
	
    private FieldFormatType formatType;
	
	private Boolean isPlasma;

	private String lastUpdateByName;

	private String createdByName;
	
	public LookupTable(Partition p,String uniqueName, String label) {
		this();
/*
		this.setPartition(p);
		this.setUniqueName(uniqueName);
		this.setLabel(label);
*/
	}

	public LookupTable() {
		super();
	}	

	public LookupTableValueType getValueType() {
		return valueType;
	}

	public void setValueType(LookupTableValueType valueType) {
		this.valueType = valueType;
	}

	public LookupTableType getType() {
		return type;
	}

	public void setType(LookupTableType type) {
		this.type = type;
	}
				
	public boolean getHideWarnings() {
		return hideWarnings == null ? false : hideWarnings.booleanValue();
	}

	public void setHideWarnings(Boolean hideWarnings) {
		this.hideWarnings = hideWarnings;
	}	

	public FieldFormatType getFormatType() {
		return formatType;
	}

	public void setFormatType(FieldFormatType formatType) {
		this.formatType = formatType;
	}
	
	// explicitly overriding to get id into response
//	public Long getLookupTableId() {
//		return getId();
//	}

	public void setLookupTableIdShouldBeIgnored(Long id) {
		// just to satisfy deserialization
	}

	public Long getNodeId() {
		return nodeId;
	}

	public void setNodeId(Number nodeId) {
		if (nodeId instanceof Long || nodeId instanceof Integer
				|| nodeId instanceof Short || nodeId instanceof Byte) {
			this.nodeId = nodeId.longValue();
		}		
	}
	
	public String getUserGroupEdit() {
		return userGroupEdit;
	}

	public void setUserGroupEdit(String userGroup) {
		this.userGroupEdit = userGroup;
	}
	
	public String getUserGroupViewDetails() {
		return userGroupViewDetails;
	}

	public void setUserGroupViewDetails(String userGroup) {
		this.userGroupViewDetails = userGroup;
	}

	public String getLastUpdateByName() {
		return lastUpdateByName;
	}

	public void setLastUpdateByName(String lastUpdateByName) {
		this.lastUpdateByName = lastUpdateByName;
	}

	public String getCreatedByName() {
		return createdByName;
	}

	public void setCreatedByName(String createdByName) {
		this.createdByName = createdByName;
	}
	
	public int getNumberOfKeyFields(){
		if (valueType != null){
			switch (valueType){
			case JSON2:
			case MATRIX2:
				return 2;
			case MATRIX3:
				return 3;
			case MATRIX4:
				return 4;
			case MATRIX5:
				return 5;
			case MATRIX6:
				return 6;				
			default:
				// do nothing
			}
		}
		return 1;
	}
}
