package net.pricefx.logic.core.config;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.fasterxml.jackson.databind.module.SimpleModule;
import net.pricefx.logic.core.client.PartitionFactory;
import net.pricefx.restapiclient.PfxClient;
import net.pricefx.restapiclient.client.AbstractPfxAuthV1;
import net.pricefx.restapiclient.client.SimplePfxAuthV1;
import okhttp3.Authenticator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.Proxy;

public class Partition extends PfxClient {
    private static final Logger log = LoggerFactory.getLogger(Partition.class);
    public static int DEFAULT_TIMEOUT = 180;

    //SSO
    public static final String STUDIO_RELAY_STATE = "PricefxStudio";

    @JsonIgnore
    protected String parentEnvironment;

    @JsonIgnore
    // should get removed once pfxpackage is using rest-api-client
    protected Proxy proxy;

    @JsonIgnore
    // should get removed once pfxpackage is using rest-api-client
    protected Authenticator proxyAuthenticator;

    public Partition(String env, String partition, String url) {
        this(env, partition, url, new SimplePfxAuthV1(partition));
    }

    public Partition(String env, String partition, String url, AbstractPfxAuthV1 authenticator) {
        super(url, partition);

        this.parentEnvironment = env;
        this.proxy = Proxy.NO_PROXY;
        this.proxyAuthenticator = null;
        authenticator(authenticator);

        timeout(DEFAULT_TIMEOUT);
    }

    public AbstractPfxAuthV1 getAuthenticator() {
        return authenticator;
    }

    @Override
    // should get removed once pfxpackage is using rest-api-client
    public PfxClient proxy(Proxy proxy) {
        super.proxy(proxy);
        this.proxy = proxy;
        return this;
    }

    @Override
    // should get removed once pfxpackage is using rest-api-client
    public PfxClient proxyAuthenticator(Authenticator proxyAuthenticator) {
        super.proxyAuthenticator(proxyAuthenticator);
        this.proxyAuthenticator = proxyAuthenticator;
        return this;
    }

    @JsonIgnore
    public String getName() {
        return partition;
    }

    public void setName(String name) {
        this.partition = name;
    }

    @JsonIgnore
    public String getUrl() {
        return url;
    }

    public void setUrl(String url) {
        this.url = url;
    }

    @JsonIgnore
    public String getParentEnvironment() {
        return parentEnvironment;
    }

    public void setParentEnvironment(String parentEnvironment) {
        this.parentEnvironment = parentEnvironment;
    }

    @JsonIgnore
    public String getNameAndEnv() {
        return parentEnvironment + "@" + partition;
    }

    public static ObjectMapper getObjectMapper(PartitionFactory partitionFactory) {
        SimpleModule module = new SimpleModule()
                .addSerializer(Partition.class, new PartitionSerializer())
                .addDeserializer(Partition.class, new PartitionDeserializer(partitionFactory));

        return new ObjectMapper()
                .configure(SerializationFeature.INDENT_OUTPUT, true)
                .registerModule(module);
    }
}