package net.pricefx.logic.core.formulamanager;

import com.fasterxml.jackson.core.type.TypeReference;
import net.pricefx.common.api.IFormulaNature;
import net.pricefx.common.api.PriceFxInterface;
import net.pricefx.logic.core.config.Partition;
import net.pricefx.logic.core.model.AbstractFormula;
import net.pricefx.logic.core.model.AbstractFormulaElement;
import net.pricefx.logic.core.model.FormulaErrorElementResult;
import net.pricefx.pckg.transform.TransformCalculationLogic;
import net.pricefx.restapiclient.PfxClient;
import net.pricefx.restapiclient.PfxClientException;
import net.pricefx.restapiclient.calls.CallWrapper;
import net.pricefx.restapiclient.model.BaseMap;
import net.pricefx.restapiclient.model.BaseRequest;
import org.jetbrains.annotations.NotNull;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import static net.pricefx.common.api.PriceFxInterface.CMD_FM_TESTEXEC;
import static net.pricefx.common.api.PriceFxInterface.CMD_WFBM_TESTEXEC;
import static net.pricefx.common.api.PriceFxInterface.PARAM_FM_NATURE;
import static net.pricefx.common.api.PriceFxInterface.PARAM_FM_NATURE_HEADERPHASES;
import static net.pricefx.common.api.PriceFxInterface.PARAM_FM_TESTFORMULA;
import static net.pricefx.common.api.PriceFxInterface.PARAM_TARGETDATE;
import static net.pricefx.logic.core.AbstractLogicSerializer.HEADER_PHASE_POSTPROCESSING;
import static net.pricefx.logic.core.AbstractLogicSerializer.HEADER_PHASE_PREPROCESSING;
import static net.pricefx.logic.core.model.AbstractFormula.ELEMENT_DEBUG;

public class TestExecute extends CallWrapper {
    public TestExecute(PfxClient client, AbstractFormula logic, Map<String, Object> params) {
        super(client, (logic.isWorkflowLogic() ? CMD_WFBM_TESTEXEC : CMD_FM_TESTEXEC) + "/", new BaseRequest(getInputData(logic, params)));
    }

    public TestExecute(PfxClient client, @NotNull String groovyExpression, @NotNull Long timeout, @NotNull Map<String, Object> paramsMap) {
        this(client, createVirtualLogic(groovyExpression, timeout), paramsMap);
    }

    public TestExecute(Partition partition, @NotNull String groovyExpression, @NotNull Map<String, Object> paramsMap) {
        this(partition, groovyExpression, (long) partition.DEFAULT_TIMEOUT, paramsMap);
    }

    private static BaseMap getInputData(AbstractFormula logic, Map<String, Object> params) {
        BaseMap inputData = new BaseMap();
        IFormulaNature nature = logic.getFormulaNatureObject();

        Object targetDate = params.remove(PARAM_TARGETDATE);
        if (targetDate == null || "".equals(targetDate)) {
            targetDate = LocalDate.now().toString();
        }
        inputData.put(PARAM_TARGETDATE, targetDate);

        if (logic.isHeaderLogic()) {
            inputData.put(PARAM_FM_NATURE, natureDataHeaderLogic(nature, params));
            inputData.putAll(params);
        }

        inputData.putAll(params);
        inputData.put(PARAM_FM_TESTFORMULA, logic.getNode());
        return inputData;
    }

    private static AbstractFormula createVirtualLogic(@NotNull String groovyExpression, @NotNull Long timeout) {
        AbstractFormula logic = AbstractFormula.createVirtualLogic(TransformCalculationLogic.DESCRIPTOR);
        AbstractFormulaElement element = logic.getElementByName(ELEMENT_DEBUG);
        element.setFormulaExpression(groovyExpression);
        element.setElementTimeout(timeout);
        return logic;
    }

    private static BaseMap natureDataHeaderLogic(IFormulaNature nature, Map<String, Object> params) {
        BaseMap map = new BaseMap();
        map.put(PriceFxInterface.PARAM_FM_NATURE_VALUE, nature.getRestApiValue());
        List<String> phases = new ArrayList<>();
        if ((Boolean) params.get(HEADER_PHASE_PREPROCESSING)) {
            phases.add(HEADER_PHASE_PREPROCESSING);
        }
        if ((Boolean) params.get(HEADER_PHASE_POSTPROCESSING)) {
            phases.add(HEADER_PHASE_POSTPROCESSING);
        }
        if (!phases.isEmpty()) {
            map.put(PARAM_FM_NATURE_HEADERPHASES, phases.toArray(new String[phases.size()]));
        }

        params.remove(HEADER_PHASE_PREPROCESSING);
        params.remove(HEADER_PHASE_POSTPROCESSING);

        return map;
    }

    public ArrayList<FormulaErrorElementResult> execute() throws PfxClientException {
        List<BaseMap> responseList = super.execute();

        try {
            return PfxClient.DEFAULT_OBJECT_MAPPER.convertValue(responseList, new TypeReference<ArrayList<FormulaErrorElementResult>>() {});
        } catch (IllegalArgumentException e) {
            throw new PfxClientException("testExecFormula: Cannot convert response to ArrayList<FormulaErrorElementResult>");
        }
    }
}
