package net.pricefx.server.dto.calculation;

import java.util.*;

import net.pricefx.common.api.CalculationResultType;
import net.pricefx.common.api.InputButtonAction;
import net.pricefx.common.api.InputType;
import net.pricefx.lmformulaengine.LmFormulaEngineContext;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;


/**
 * This class is used to transport a configurator entry
 * 
 * @author cat
 *
 */
@JsonIgnoreProperties(ignoreUnknown=true)
public class ConfiguratorEntry implements ComplexCalculationResult {

    private static final long serialVersionUID = -5268805022880613L;
    
    private List<ContextParameter> inputs;
	private String message;
	private List<InputButtonAction> hiddenActions;
	private LmFormulaEngineContext ctx;
	private Map<String,String> additionalConfig;
	
	public ConfiguratorEntry() {}
	
	public List<ContextParameter> getInputs() {
		return inputs;
	}
	
	public ContextParameter getFirstInput() {
		return inputs != null ? inputs.get(0) : null;
	}	

	public void setInputs(List<ContextParameter> inputs) {
		this.inputs = inputs;
	}

	public String getMessage() {
		return message;
	}

	public void setMessage(String message) {
		this.message = message;
	}
	
	public void setHiddenActions(InputButtonAction... hiddenActions) {
		this.hiddenActions = hiddenActions == null ? null : Arrays.asList(hiddenActions);
	}
	
	public List<InputButtonAction> getHiddenActions() {
		return hiddenActions;
	}
	
	public ContextParameter createParameter(InputType type, String name) {
		return createParameter(type, name, null);
	}
	
	public ContextParameter createParameter(InputType type, String name, Date targetDate) {
		ContextParameter input = new ContextParameter(type, name, targetDate, null);
		
		if(ctx!= null) input.setValue(ctx.getParameter(name));
		
		if(inputs == null) inputs = new ArrayList<ContextParameter>();
		inputs.add(input);
		
		return input;
	}

	public ConfiguratorEntry createParameter(ContextParameter input, groovy.lang.Closure defaultValue) {
		if(inputs == null) inputs = new ArrayList<ContextParameter>();

		if(ctx!= null) input.setValue(ctx.getParameter(input.getName()));
		if(input.getValue() == null) input.setValue(defaultValue.call());

		inputs.add(input);
		return this;
	}

	public ConfiguratorEntry addParameter(ContextParameter input) {
		if(inputs == null) inputs = new ArrayList<ContextParameter>();
		inputs.add(input);
		return this;
	}

	public Map<String, String> getAdditionalConfig() {
		return additionalConfig;
	}

	public void setAdditionalConfig(Map<String, String> additionalConfig) {
		this.additionalConfig = additionalConfig;
	}

	public void addAdditionalConfigValue(String name, String value) {
		if(additionalConfig == null)
			additionalConfig = new HashMap<String,String>();
		additionalConfig.put(name,value);
	}

	@Override
	public String getResultType() {
		return CalculationResultType.CONFIGURATORENTRY.name();
	}

	@JsonIgnore
	public void setContext(LmFormulaEngineContext ctx) {
		this.ctx = ctx;		
	}
}
